@@ -1,8 +1,10 @@
 require_relative 'timing_buffer_shared'
+require 'concurrent/channel/buffer/timer'
+require 'concurrent/atomic/atomic_boolean'
 
 module Concurrent::Channel::Buffer
 
-  RSpec.describe Timer, edge: true, notravis: true do
+  RSpec.describe Timer, edge: true do
 
     let(:delay) { 0.1 }
     subject { described_class.new(0.1) }
