@@ -1,5 +1,5 @@
 require 'concurrent/edge/promises'
-require 'thread'
+require 'concurrent/atomic/count_down_latch'
 
 RSpec.describe 'Concurrent::Promises' do
 
@@ -157,6 +157,24 @@ RSpec.describe 'Concurrent::Promises' do
 
       expect(any.value!).to eq :value
     end
+
+    it 'treats a resolved Event as a fulfilled Future' do
+        any = any_fulfilled_future(
+          resolved_event,
+          fulfilled_future(:value),
+        )
+
+        expect(any.value!).to eq nil
+    end
+
+    it 'treats a pending Event as a pending Future' do
+        any = any_fulfilled_future(
+          resolvable_event,
+          fulfilled_future(:value),
+        )
+
+        expect(any.value!).to eq :value
+    end
   end
 
   describe '.zip' do
@@ -362,12 +380,12 @@ RSpec.describe 'Concurrent::Promises' do
     end
 
     it 'chains' do
-      future0 = future { 1 }.then { |v| v + 2 } # both executed on default FAST_EXECUTOR
-      future1 = future0.then_on(:fast) { raise 'boo' } # executed on IO_EXECUTOR
-      future2 = future1.then { |v| v + 1 } # will reject with 'boo' error, executed on default FAST_EXECUTOR
-      future3 = future1.rescue { |err| err.message } # executed on default FAST_EXECUTOR
-      future4 = future0.chain { |success, value, reason| success } # executed on default FAST_EXECUTOR
-      future5 = future3.with_default_executor(:fast) # connects new future with different executor, the new future is resolved when future3 is
+      future0 = future { 1 }.then { |v| v + 2 } # both executed on default IO_EXECUTOR
+      future1 = future0.then_on(:fast) { raise 'boo' } # executed on FAST_EXECUTOR
+      future2 = future1.then { |v| v + 1 } # will reject with 'boo' error, executed on FAST_EXECUTOR
+      future3 = future1.rescue { |err| err.message } # executed on FAST_EXECUTOR
+      future4 = future0.chain { |success, value, reason| success } # executed on FAST_EXECUTOR
+      future5 = future3.with_default_executor(:io) # connects new future with different executor, the new future is resolved when future3 is
       future6 = future5.then(&:capitalize) # executes on IO_EXECUTOR because default was set to :io on future5
       future7 = future0 & future3
       future8 = future0.rescue { raise 'never happens' } # future0 fulfills so future8'll have same value as future 0
@@ -384,12 +402,12 @@ RSpec.describe 'Concurrent::Promises' do
       expect(table.join("\n")).to eq <<-TABLE.gsub(/^\s+\|/, '').strip
         |index success      value reason pool d.pool
         |    0    true          3          io     io
-        |    1   false               boo fast     io
-        |    2   false               boo   io     io
-        |    3    true        boo          io     io
+        |    1   false               boo fast   fast
+        |    2   false               boo fast   fast
+        |    3    true        boo        fast   fast
         |    4    true       true          io     io
-        |    5    true        boo               fast
-        |    6    true        Boo        fast   fast
+        |    5    true        boo                 io
+        |    6    true        Boo          io     io
         |    7    true [3, "boo"]                 io
         |    8    true          3          io     io
       TABLE
@@ -560,6 +578,18 @@ RSpec.describe 'Concurrent::Promises' do
       expect(event.wait(0, true)).to be_truthy
     end
 
+    specify "#resolve(raise_on_reassign = true)" do
+      event = resolvable_event
+      event.resolve
+      expect { event.resolve }.to raise_error(Concurrent::MultipleAssignmentError)
+    end
+
+    specify "#resolve(raise_on_reassign = false)" do
+      event = resolvable_event
+      event.resolve
+      expect(event.resolve(false)).to be_falsey
+    end
+
     specify "reservation" do
       event = resolvable_event
       expect(event.reserve).to be_truthy
@@ -681,7 +711,8 @@ RSpec.describe 'Concurrent::Promises' do
   end
 
   describe 'interoperability' do
-    it 'with processing actor', if: !defined?(JRUBY_VERSION) do
+    it 'with processing actor', if: !Concurrent.on_jruby? do
+      require 'concurrent/actor'
       actor = Concurrent::Actor::Utils::AdHoc.spawn :doubler do
         -> v { v * 2 }
       end
@@ -692,20 +723,20 @@ RSpec.describe 'Concurrent::Promises' do
           value!).to eq 6
     end
 
-    if Concurrent.const_defined? :ErlangActor
-      it 'with erlang actor' do
-        actor = Concurrent::ErlangActor.spawn type: :on_thread do
-          reply receive * 2
-        end
-
-        expect(future { 2 }.
-            then_ask(actor).
-            then { |v| v + 2 }.
-            value!).to eq 6
+    it 'with erlang actor' do
+      require 'concurrent/edge/erlang_actor'
+      actor = Concurrent::ErlangActor.spawn type: :on_thread do
+        reply receive * 2
       end
+
+      expect(future { 2 }.
+          then_ask(actor).
+          then { |v| v + 2 }.
+          value!).to eq 6
     end
 
     it 'with channel' do
+      require 'concurrent/edge/channel'
       ch1 = Concurrent::Promises::Channel.new
       ch2 = Concurrent::Promises::Channel.new
 
@@ -723,77 +754,32 @@ RSpec.describe 'Concurrent::Promises' do
   specify 'zip_futures_over' do
     expect(zip_futures_over([1, 2]) { |v| v.succ }.value!).to eq [2, 3]
   end
-end
 
-RSpec.describe 'Concurrent::ProcessingActor' do
-  specify do
-    actor = Concurrent::ProcessingActor.act do |the_actor|
-      the_actor.receive.then do |message|
-        # the actor ends with message
-        message
-      end
-    end #
-
-    actor.tell! :a_message
-    expect(actor.termination.value!).to eq :a_message
-
-    def count(actor, count)
-      # the block passed to receive is called when the actor receives the message
-      actor.receive.then do |number_or_command, answer|
-        # number_or_command, answer = p a
-        # p number_or_command, answer
-
-        # code which is evaluated after the number is received
-        case number_or_command
-        when :done
-          # this will become the result (final value) of the actor
-          count
-        when :count
-          # reply the current count
-          answer.fulfill count
-          # continue running
-          count(actor, count)
-        when Integer
-          # this will call count again to set up what to do on next message, based on new state `count + numer`
-          count(actor, count + number_or_command)
+  describe 'value!' do
+    %w[with without].each do |timeout|
+      it "does not return spuriously #{timeout} timeout" do
+        # https://github.com/ruby-concurrency/concurrent-ruby/issues/1015
+        trapped = false
+        original_handler = Signal.trap(:SIGHUP) { trapped = true }
+        begin
+          task = Concurrent::Promises.future { sleep }
+          main = Thread.current
+          t = Thread.new do
+            Thread.pass until main.stop?
+            Process.kill(:SIGHUP, Process.pid)
+            sleep 0.1
+            main.raise "Done"
+          end
+          expect {
+            timeout == 'with' ? task.value!(3600) : task.value!
+            expect(task).to be_resolved # fail if #value! returned
+          }.to raise_error(RuntimeError, "Done")
+          expect(trapped).to be true
+          t.join
+        ensure
+          Signal.trap(:SIGHUP, original_handler)
         end
       end
-      # evaluation of count ends immediately
-      # code which is evaluated before the number is received, should be empty
     end
-
-    counter = Concurrent::ProcessingActor.act { |a| count a, 0 }
-    answer  = counter.tell!(2).ask_op { |a| [:count, a] }.value!
-    expect(counter.tell!(3).tell!(:done).termination.value!).to eq 5
-    expect(answer.value!).to eq 2
-
-    add_once_actor = Concurrent::ProcessingActor.act do |the_actor|
-      the_actor.receive.then do |a, b, reply|
-        result = a + b
-        reply.fulfill result
-        # terminate with result value
-        result
-      end
-    end
-
-    expect(add_once_actor.ask_op { |a| [1, 2, a] }.value!.value!).to eq 3
-    # expect(add_once_actor.ask_operation(%w(ab cd)).reason).to be_a_kind_of RuntimeError
-    expect(add_once_actor.termination.value!).to eq 3
-
-    def pair_adder(actor)
-      (actor.receive & actor.receive).then do |(value1, answer1), (value2, answer2)|
-        result = value1 + value2
-        answer1.fulfill result if answer1
-        answer2.fulfill result if answer2
-        pair_adder actor
-      end
-    end
-
-    pair_adder = Concurrent::ProcessingActor.act { |a| pair_adder a }
-    pair_adder.ask_op { |a| [2, a] }
-    answer = pair_adder.ask_op { |a| [3, a] }.value!
-    expect(answer.value!).to eq 5
-    expect((pair_adder.ask_op { |a| ['a', a] }.value! & pair_adder.ask_op { |a| ['b', a] }.value!).value!).to eq %w[ab ab]
-    expect((pair_adder.ask_op { |a| ['a', a] }.value! | pair_adder.ask_op { |a| ['b', a] }.value!).value!).to eq 'ab'
   end
 end
